==> jungle-1.0.tar.gz: jungle-1.0/jungle/apps/timereport/vacation_form.py <==
# -*- coding: latin-1 -*-
from django.newforms.models import form_for_model
from django.newforms import fields, widgets
from jungle.forms import fields as jfields
from jungle.templatetags.jungle_tags import ugettext as _
from models import Transit, Iter
from jungle.ref.models import WeekDay 
import django.newforms as forms
from datetime import *
import re
from jungle.functions.utils import Dict2Obj
from jungle.middleware.thread_local import get_current_user, _thread_locals
from jungle.timereport.time_display import HOLIDAYS
from django.conf import settings

user = get_current_user
VACATION_MSG = getattr(settings, 'TIMEREPORT_VACATION_MSG',
                       _("your vacation request has been received"))
## reasons that can happen all days
#REASON_ALL_DAYS = ('0000','T')

class VacationForm(form_for_model(Transit)):

    def __init__(self, *args, **kwargs):
        super(VacationForm, self).__init__(*args, **kwargs)

#         wdg_time = {'required' : False, 'widget' : forms.TextInput(
#             attrs={'size' : 8,'maxlength' : 8, 'class' : 'vTimeField',})}
        wdg_time = {'required' : False}
        wdg_date = {'required' : False, 'widget' : forms.TextInput(
            attrs={'size' : 10,'maxlength' : 10, 'class' : 'vDateField',})}
        
        self.fields['start_time'] = jfields.JungleTimeField(**wdg_time )
        self.fields['end_time'] = jfields.JungleTimeField(**wdg_time)
        self.fields['start_date'] = jfields.JungleDateField(**{'required':False, })
        self.fields['end_date'] = jfields.JungleDateField(**{'required':False, })
        self.fields['date_reference'] = jfields.JungleDateField(**wdg_date)
        self.fields['description'] = fields.CharField(max_length=100)
        self.fields['next'] = fields.CharField(max_length=100, required=False,
                                               widget=widgets.HiddenInput)
        

        self.fields['mode'] = fields.ChoiceField(
            choices=(('-----',''),('days','giorni'), ('hours','ore')))
        for f in ( 'status','date_time','direction','profile','type',):
            self.fields[f].required = False

    def save(self):
        """register holiday and permit for each (working) day
        reason is choosen in the form as well as user
        """
        self.profile = self.cd.user.get_profile().time_profile
        self.ProfileDetails = self.profile.get_my_details()

        self.iter_obj = self.save_description()
        
        ## if the reason says it is 'worked' we assume it can be booked also for
        ## a nationa feasts of a week day that is not in the empliyee profile
        ## on the contrary illness and holiday will skip throught such days
        for day in self.get_working_days(self.profile):
            if day in HOLIDAYS and not self.cd.reason.worked == 'w':
                continue
            
            if self.cd.mode == "days":
                ## if start/end time are provided, we don't look at profile
                ## and we only use 1 block
                if self.cd.start_time and self.cd.end_time:
                    date_time_in = datetime.combine(day, self.cd.start_time)
                    date_time_out = datetime.combine(day, self.cd.end_time)
                    self.add_transit(day, date_time_in, 1)     ## IN
                    self.add_transit(day, date_time_out, 4)  ## OUT
                else:
                    for odv, detail in self.ProfileDetails[day.isoweekday()].items():
                        if not detail:
                            continue
                        date_time_in = datetime.combine(day, detail.time)
                        date_time_out = date_time_in +timedelta(seconds=detail.duration)
                        self.add_transit(day, date_time_in, odv)     ## IN
                        self.add_transit(day, date_time_out, odv+1)  ## OUT

            else:
                date_time_in  =  datetime.combine(day, self.cd.start_time)
                date_time_out =  datetime.combine(day, self.cd.end_time) 
                ## 'P' is considered an extra so is displayed in an extra colum (11)
                ## any other is displayed according to the time
                ## FIXME: if a permit is all day long or similar should not be
                ## rendere in the extra column
                if self.cd.reason.code == 'P':
                    odv = 11
                else:
                    if self.cd.start_time < time(12,0):
                        odv = 1
                    else:
                        odv = 3
                
                self.add_transit(day, date_time_in, odv)   ## IN 
                self.add_transit(day, date_time_out, odv+1)  ## OUT

        self.vacation_acknoledgment()
        return self.iter_obj
            
    def add_transit(self, day, date_time, odv):
        Transit(
            status=2,  # manually inserted
            reason=self.cd.reason,  # holiday, permit, transfert
            user=self.cd.user,
            date_reference=day,
            date_time=date_time,
            direction='0',  # no transit
            type=4,  # transito con causale
            badge_reader_id = None,
            profile = self.profile,
            odv=odv,
            iter_id=self.iter_obj.id,
            ).save()

    def vacation_acknoledgment(self):
        if self.cd.reason.code == 'F':
            self.cd.user.email_user(_('Vacation request'),VACATION_MSG )
                                    
    def save_description(self):
        descr = Iter(status=1, description=self.cd.description,
                     reason=self.cd.reason, user=self.cd.user,
                     date_start=self.cd.start_date, date_end=self.cd.end_date)
        descr.save()
        return descr
        

    def get_working_days(self, profile):
        """list all working days from 'start' to 'end' for the profile 'profile'
        skipping holidays
        """
        days = []
        date = self.cd.start_date
        if not self.cd.end_date:
            self.cleaned_data['end_date'] = self.cd.start_date
            
        while date <=  self.cd.end_date:
            days += [date]
            date += timedelta(days=1)
        return days

    def clean(self):

        self.cd = Dict2Obj(self.cleaned_data)
        msg = ''
        try:
            if self.cd.start_date and not self.cd.end_date:
                self.cleaned_data['end_date'] = self.cd.start_date

            if not self.cd.mode:
                msg = u'Devi indicare se vuoi la modalit ore o giorni. <br>'

            if not self.cd.user:
                msg = u'Indicare il dipendente. <br>'
            if self.cd.mode == 'days':
                if not self.cd.start_date:
                    msg += u'Manca la data di inizio della vacanza. <br>'

                if not self.cd.end_date:
                    msg += u'Manca la data di fine della vacanza. <br>'

                if self.cd.end_date < self.cd.start_date:
                    msg += u'La data finale deve essere successiva a quella iniziale. <br>'
            else:
                if 'start_time' not in  self.cleaned_data:
                    msg += "Manca l'ora di inizio del permesso. <br>"

                if 'end_time' not in  self.cleaned_data:
                    msg += u"Manca l'ora di fine del permesso. "

                if self.cd.end_time < self.cd.start_time:
                    msg += u"L'ora finale deve essere successiva a quella iniziale. <br>"

        ## if we get a key error it's probably becouse a data conversion was not
        ## correctly finisced and we already have a ValudationError from
        ## full_clean method 
        except KeyError:
            pass
        
        if msg:
            raise forms.ValidationError(msg)
        return self.cleaned_data
             
#    def clean_1i1(self):
#         print "TYPE", type(self.cleaned_data.get('1i1'))
#         raise ValidationError(u'Entri a quest ora? type:-%s-' %
#                                 self.cleaned_data.get('1i1'))

==> jungle-1.0.2.tar.gz: jungle-1.0.2/jungle/apps/timereport/vacation_form.py <==
# -*- coding: latin-1 -*-
from django.newforms.models import form_for_model
from django.newforms import fields, widgets
from jungle.forms import fields as jfields
from jungle.templatetags.jungle_tags import ugettext as _
from models import Transit, Iter
from jungle.ref.models import WeekDay
import django.newforms as forms
from datetime import *
import re
from jungle.functions.utils import Dict2Obj
from jungle.middleware.thread_local import get_current_user, _thread_locals
from jungle.timereport.time_display import HOLIDAYS
from django.conf import settings

user = get_current_user
VACATION_MSG = getattr(settings, 'TIMEREPORT_VACATION_MSG',
                       _("your vacation request has been received"))
## reasons that can happen all days
#REASON_ALL_DAYS = ('0000','T')

class VacationForm(form_for_model(Transit)):

    def __init__(self, *args, **kwargs):
        super(VacationForm, self).__init__(*args, **kwargs)

#         wdg_time = {'required' : False, 'widget' : forms.TextInput(
#             attrs={'size' : 8,'maxlength' : 8, 'class' : 'vTimeField',})}
        wdg_time = {'required': False}
        wdg_date = {'required': False, 'widget': forms.TextInput(
            attrs={'size': 10, 'maxlength': 10, 'class': 'vDateField', })}

        self.fields['start_time'] = jfields.JungleTimeField(**wdg_time)
        self.fields['end_time'] = jfields.JungleTimeField(**wdg_time)
        self.fields['start_date'] = jfields.JungleDateField(**{'required': False, })
        self.fields['end_date'] = jfields.JungleDateField(**{'required': False, })
        self.fields['date_reference'] = jfields.JungleDateField(**wdg_date)
        self.fields['description'] = fields.CharField(max_length=100)
        self.fields['next'] = fields.CharField(max_length=100, required=False,
                                               widget=widgets.HiddenInput)

        self.fields['mode'] = fields.ChoiceField(
            choices=(('-----', ''), ('days', 'giorni'), ('hours', 'ore')))
        for f in ('status', 'date_time', 'direction', 'profile', 'type',):
            self.fields[f].required = False

    def save(self):
        """register holiday and permit for each (working) day
        reason is choosen in the form as well as user
        """
        self.profile = self.cd.user.get_profile().time_profile
        self.ProfileDetails = self.profile.get_my_details()

        self.iter_obj = self.save_description()

        ## if the reason says it is 'worked' we assume it can be booked also for
        ## a nationa feasts of a week day that is not in the empliyee profile
        ## on the contrary illness and holiday will skip throught such days
        for day in self.get_working_days(self.profile):
            if day in HOLIDAYS and not self.cd.reason.worked == 'w':
                continue

            if self.cd.mode == "days":
                ## if start/end time are provided, we don't look at profile
                ## and we only use 1 block
                if self.cd.start_time and self.cd.end_time:
                    date_time_in = datetime.combine(day, self.cd.start_time)
                    date_time_out = datetime.combine(day, self.cd.end_time)
                    self.add_transit(day, date_time_in, 1)  # IN
                    self.add_transit(day, date_time_out, 4)  # OUT
                else:
                    for odv, detail in self.ProfileDetails[day.isoweekday()].items():
                        if not detail:
                            continue
                        date_time_in = datetime.combine(day, detail.time)
                        date_time_out = date_time_in + timedelta(seconds=detail.duration)
                        self.add_transit(day, date_time_in, odv)  # IN
                        self.add_transit(day, date_time_out, odv+1)  # OUT

            else:
                date_time_in = datetime.combine(day, self.cd.start_time)
                date_time_out = datetime.combine(day, self.cd.end_time)
                ## 'P' is considered an extra so is displayed in an extra colum (11)
                ## any other is displayed according to the time
                ## FIXME: if a permit is all day long or similar should not be
                ## rendere in the extra column
                if self.cd.reason.code == 'P':
                    odv = 11
                else:
                    if self.cd.start_time < time(12, 0):
                        odv = 1
                    else:
                        odv = 3

                self.add_transit(day, date_time_in, odv)  # IN
                self.add_transit(day, date_time_out, odv+1)  # OUT

        self.vacation_acknoledgment()
        return self.iter_obj

    def add_transit(self, day, date_time, odv):
        Transit(
            status=2,  # manually inserted
            reason=self.cd.reason,  # holiday, permit, transfert
            user=self.cd.user,
            date_reference=day,
            date_time=date_time,
            direction='0',  # no transit
            type=4,  # transito con causale
            badge_reader_id=None,
            profile=self.profile,
            odv=odv,
            iter_id=self.iter_obj.id,
        ).save()

    def vacation_acknoledgment(self):
        if self.cd.reason.code == 'F':
            self.cd.user.email_user(_('Vacation request'), VACATION_MSG)

    def save_description(self):
        descr = Iter(status=1, description=self.cd.description,
                     reason=self.cd.reason, user=self.cd.user,
                     date_start=self.cd.start_date, date_end=self.cd.end_date)
        descr.save()
        return descr

    def get_working_days(self, profile):
        """list all working days from 'start' to 'end' for the profile 'profile'
        skipping holidays
        """
        days = []
        date = self.cd.start_date
        if not self.cd.end_date:
            self.cleaned_data['end_date'] = self.cd.start_date

        while date <= self.cd.end_date:
            days += [date]
            date += timedelta(days=1)
        return days

    def clean(self):

        self.cd = Dict2Obj(self.cleaned_data)
        msg = ''
        try:
            if self.cd.start_date and not self.cd.end_date:
                self.cleaned_data['end_date'] = self.cd.start_date

            if not self.cd.mode:
                msg = u'Devi indicare se vuoi la modalit ore o giorni. <br>'

            if not self.cd.user:
                msg = u'Indicare il dipendente. <br>'
            if self.cd.mode == 'days':
                if not self.cd.start_date:
                    msg += u'Manca la data di inizio della vacanza. <br>'

                if not self.cd.end_date:
                    msg += u'Manca la data di fine della vacanza. <br>'

                if self.cd.end_date < self.cd.start_date:
                    msg += u'La data finale deve essere successiva a quella iniziale. <br>'
            else:
                if 'start_time' not in self.cleaned_data:
                    msg += "Manca l'ora di inizio del permesso. <br>"

                if 'end_time' not in self.cleaned_data:
                    msg += u"Manca l'ora di fine del permesso. "

                if self.cd.end_time < self.cd.start_time:
                    msg += u"L'ora finale deve essere successiva a quella iniziale. <br>"

        ## if we get a key error it's probably becouse a data conversion was not
        ## correctly finisced and we already have a ValudationError from
        ## full_clean method
        except KeyError:
            pass

        if msg:
            raise forms.ValidationError(msg)
        return self.cleaned_data

#    def clean_1i1(self):
#         print "TYPE", type(self.cleaned_data.get('1i1'))
#         raise ValidationError(u'Entri a quest ora? type:-%s-' %
#                                 self.cleaned_data.get('1i1'))
